--
-- Copyright (c) 2021 lalawue
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--
local fType = type
local fNext = next
local fString = tostring
local fAssert = assert
local fOpen = io.open
local pairs = pairs
local ipairs = ipairs
local Utils = {}
do
	local __stype__ = nil
	local __clsname__ = "Utils"
	local __clstype__ = Utils
	__clstype__.typename = __clsname__
	__clstype__.typekind = 'class'
	__clstype__.classtype = __clstype__
	__clstype__.supertype = __stype__
	__clstype__.isKindOf = function(cls, a) return a and ((cls.classtype == a) or (cls.supertype and cls.supertype:isKindOf(a))) or false end
	-- declare var and methods
	function __clstype__.stringValue(str)
		local first = str:sub(1, 1)
		if first == '"' or first == "'" or first == '[' then
			return str
		else
			return '"' .. str .. '"'
		end
	end
	function __clstype__.serializeTable(t, p, c, s)
		local n = 0
		for i, v in fNext, t do
			n = n + 1
		end
		local ti = 1
		local e = n > 0
		local str = ""
		local _table = Utils.serializeTable
		c = c or {  }
		p = p or 1
		s = s or string.rep
		local function _format(v, is_table)
			return is_table and (c[v][2] >= p) and _table(v, p + 1, c, s) or (fType(v) == "string" and Utils.stringValue(v) or fString(v))
		end
		c[t] = { t, 0 }
		for i, v in fNext, t do
			local typ_i, typ_v = fType(i) == 'table', fType(v) == 'table'
			c[i], c[v] = (not c[i] and typ_i) and { i, p } or c[i], (not c[v] and typ_v) and { v, p } or c[v]
			str = str .. s('  ', p) .. '[' .. _format(i, typ_i) .. '] = ' .. _format(v, typ_v) .. (ti < n and ',' or '') .. '\n'
			ti = ti + 1
		end
		return ('{' .. (e and '\n' or '')) .. str .. (e and s('  ', p - 1) or '') .. '}'
	end
	function __clstype__.split(self, sep, max, regex)
		fAssert(sep ~= "")
		fAssert(max == nil or max >= 1)
		local record = {  }
		if self:len() > 0 then
			local plain = not regex
			max = max or -1
			local field, start = 1, 1
			local first, last = self:find(sep, start, plain)
			while first and max ~= 0 do
				record[field] = self:sub(start, first - 1)
				field = field + 1
				start = last + 1
				first, last = self:find(sep, start, plain)
				max = max - 1
			end
			record[field] = self:sub(start)
		else
			record[1] = ""
		end
		return record
	end
	function __clstype__.set(tbl)
		local s = {  }
		for _, v in ipairs(tbl) do
			s[v] = true
		end
		return s
	end
	-- declare after set()
	__clstype__.blank_set = Utils.set({ " ", "\t", "\n", "\r" })
	function __clstype__.trim(self)
		local i = 1
		local j = self:len()
		local blank_set = Utils.blank_set
		while i <= j do
			if blank_set[self:sub(i, i)] then
				i = i + 1
			elseif blank_set[self:sub(j, j)] then
				j = j - 1
			else
				return self:sub(i, j)
			end
		end
		return self
	end
	function __clstype__.seqReduce(tbl, init, func)
		for i, v in ipairs(tbl) do
			init = func(init, i, v)
		end
		return init
	end
	__clstype__.read_option = _VERSION == "Lua 5.1" and "*a" or "a"
	function __clstype__.readFile(file_path)
		local f, err = fOpen(file_path, "rb")
		if not f then
			return nil, err
		end
		local data = f:read(Utils.read_option)
		f:close()
		return data
	end
	function __clstype__.writeFile(file_path, content)
		local f = fOpen(file_path, "wb")
		if not f then
			return 
		end
		f:write(content)
		f:close()
		return true
	end
	function __clstype__.copy(it)
		local ot = {  }
		for k, v in pairs(it) do
			ot[k] = v
		end
		return ot
	end
	function __clstype__.suffix(str)
		for i = str:len(), 1, -1 do
			if str:sub(i, i) == '.' then
				return str:sub(i + 1, str:len())
			end
		end
		return ""
	end
	function __clstype__.debug(str)
		io.write(str .. "\n")
	end
	function __clstype__.dump(t)
		Utils.debug(Utils.serializeTable(t))
	end
	-- position line in content
	function __clstype__.posLine(content, lpos, cpos)
		fAssert(fType(content) == "string", "Invalid content")
		fAssert(fType(lpos) == "number", "Invalid pos")
		local ln_num = 1
		for _ in content:sub(1, lpos):gmatch("\n") do
			ln_num = ln_num + 1
		end
		local num = ln_num
		local ln_content = ""
		for line in content:gmatch("([^\n]*)\n?") do
			if num == 1 then
				ln_content = line
				break
			end
			num = num - 1
		end
		return { line = ln_num, message = ln_content }
	end
	-- declare end
	local __ins_mt__ = {
		__tostring = function() return "instance of " .. __clsname__ end,
		__index = function(t, k)
			local v = __clstype__[k]
			if v ~= nil then rawset(t, k, v) end
			return v
		end,
	}
	setmetatable(__clstype__, {
		__tostring = function() return "class " .. __clsname__ end,
		__index = function(_, k)
			local v = __stype__ and __stype__[k]
			if v ~= nil then rawset(__clstype__, k, v) end
			return v
		end,
		__call = function(_, ...)
			local ins = setmetatable({}, __ins_mt__)
			return ins
		end,
	})
end
return Utils
